/*
 +---------------------------------------------------------------------+
 | csocket.cc                                                          |
 | implements CSocket classs                                           |
 |                                                                     |
 | Author:  Bob Pill, NCIS                                             |
 |          bobp@newcorridor.com                                       |
 |                                                                     |
 | Created: Sept 15, 1997                                              |
 |                                                                     |
 | Copyright (c) 1997 New Corridor Information Services.               |
 | All rights reserved.                                                |
 |                                                                     |
 |                                                                     |
 | THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR        |
 | IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED      |
 | WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR          |
 | PURPOSE.                                                            |
 |                                                                     |
 |---------------------------------------------------------------------|
 |  ***Revision-History***  Begin-Log                                  |
 |     Initial creation - (started at ver 2.0.1) Sept 15, 1997 (BP)    |
 |     Version 2.1.0 update Jan 01, 1998 (BP)                          |
 |     Version 2.1.1 update Feb 03, 1998 (BP)                          |
 |     Version 2.1.2 update May 17, 1998 (BP)                          |
 |     Version 2.1.3 update Aug 04, 1998 (BP)                          |
 |  ***Revision-History***  End-Log                                    |
 +---------------------------------------------------------------------+
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <memory.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include "stddef.h"
#include "csocket.h"


// CSocket class

// constructor
CSocket::CSocket(void)
{
    Init();
}


// constructor
CSocket::CSocket(const char *pszSocketName)
{
    strcpy(m_szSocketPath,pszSocketName);
    Init();
}


// destructor
CSocket::~CSocket()
{
    if(m_fdSocketServer > 0)
      close(m_fdSocketServer);
    if(m_fdSocketNormal > 0)
      close(m_fdSocketNormal);
    if(strlen(m_szSocketPath))
      unlink(m_szSocketPath);
}


void CSocket::Init(void)
{
    m_fdSocketNormal = -1;
    m_fdSocketServer = -1;
    memset(&m_saunLocal,0,sizeof(m_saunLocal));
    m_iLocalAddrLen = 0;
    memset(&m_saunRemote,0,sizeof(m_saunLocal));
    m_iRemoteAddrLen = 0;
}


int CSocket::Accept(void)
{
    int iRetCode = RC_SUCCESS;

    m_fdSocketNormal = accept(m_fdSocketServer,(struct sockaddr *)&m_saunRemote,&m_iRemoteAddrLen);
    if(m_fdSocketNormal < 0)
      iRetCode = RC_SOCKET_ACCEPT_FAILED;
    return(iRetCode);
}


int CSocket::Listen(void)
{
    int iRetCode = RC_SUCCESS;

    if((m_fdSocketServer = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
    {
      iRetCode = RC_SOCKET_CREATE_FAILED;
      goto listen_exit;
    }

    unlink(m_szSocketPath);  // remove in case it already exists

    // fill in socket address structure
    memset(&m_saunLocal,0,sizeof(m_saunLocal));
    m_saunLocal.sun_family = AF_UNIX;
    strcpy(m_saunLocal.sun_path,m_szSocketPath);
    m_iLocalAddrLen = sizeof(m_saunLocal.sun_len) 
                    + sizeof(m_saunLocal.sun_family)
                    + sizeof(m_saunLocal.sun_path)
                    + 1;
    m_saunLocal.sun_len = m_iLocalAddrLen;

    // bind socket name to descriptor
    if(bind(m_fdSocketServer, (struct sockaddr *)&m_saunLocal,m_iLocalAddrLen) < 0)
    {
      iRetCode = RC_SOCKET_BIND_FAILED;
      goto listen_exit;
    }

    // server listen on socket
    if(listen(m_fdSocketServer, LISTEN_MAX_BACKLOG))
      iRetCode = RC_SOCKET_LISTEN_FAILED;
           
listen_exit:

    return(iRetCode);
}



int CSocket::Connect(char *pszServerSocket)
{
    int iRetCode = RC_SUCCESS;

    if((m_fdSocketNormal = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
    {
      iRetCode = RC_SOCKET_CREATE_FAILED;
      goto connect_exit;
    }

    // fill in socket address structure
    memset(&m_saunRemote,0,sizeof(m_saunRemote));
    m_saunRemote.sun_family = AF_UNIX;
    strcpy(m_saunRemote.sun_path,pszServerSocket);
    m_iRemoteAddrLen = sizeof(m_saunRemote.sun_len)
                     + sizeof(m_saunRemote.sun_family)
                     + sizeof(m_saunRemote.sun_path)
                     + 1;

    m_saunRemote.sun_len = m_iRemoteAddrLen;

    // connect to server socket
    if(connect(m_fdSocketNormal, (struct sockaddr *)&m_saunRemote,m_iRemoteAddrLen) < 0)
      iRetCode = RC_SOCKET_CONNECT_FAILED;
   
connect_exit:

    return iRetCode;
}


int CSocket::Close(void)
{
    if(m_fdSocketServer > 0)
    {
      close(m_fdSocketServer);
      m_fdSocketServer = -1;
    }
    return(RC_SUCCESS);
}    


int CSocket::Release(void)
{
    if(m_fdSocketNormal > 0)
    {
      close(m_fdSocketNormal);
      m_fdSocketNormal = -1;
    }
    return(RC_SUCCESS);
}    


int CSocket::Read(void)
{
    int iRetCode = RC_SUCCESS;
    int iBytesRead = 0;
    char *pBuffer = m_bufRcve.GetBuffer();

    iBytesRead = read(m_fdSocketNormal,pBuffer,MAX_BUFFERSIZE);
    if(iBytesRead < 0)
      iRetCode = RC_SOCKET_READ_ERROR;
    else
      pBuffer[iBytesRead] = '\0';
    m_bufRcve.SetLength(iBytesRead);
    return(iRetCode);
}


int CSocket::Read(int iBytesToRead)
{
    int iRetCode = RC_SUCCESS;
    int iBytesRemain = iBytesToRead;
    int iBytesRead = 0;
    int iBytesTotal = 0;
    char *pBuffer = m_bufRcve.GetBuffer();

    while(iBytesRemain > 0)
    {
      iBytesRead = read(m_fdSocketNormal,pBuffer,iBytesRemain);
      if(iBytesRead < 0)
      {
        iRetCode = RC_SOCKET_READ_ERROR;
        break;
      }
      else if(iBytesRead == 0)
        break;

      iBytesTotal += iBytesRead;
      iBytesRemain -= iBytesRead;
      pBuffer += iBytesRead;
    }
    m_bufRcve.SetLength(iBytesTotal);
    return(iRetCode);
}




int CSocket::Write(void)
{
    int iRetCode = RC_SUCCESS;
    int iBytesRemain = m_bufSend.GetLength();
    int iBytesWritten = 0;
    char *pBuffer = m_bufSend.GetBuffer();

    while(iBytesRemain > 0)
    {
      iBytesWritten = write(m_fdSocketNormal,pBuffer,iBytesRemain);
      if(iBytesWritten <= 0)
      {
        iRetCode = RC_SOCKET_WRITE_ERROR;
        break;
      }

      iBytesRemain -= iBytesWritten;
      pBuffer += iBytesWritten;
    }
    return(iRetCode);
}



// CBuffer class

// constructor
CBuffer::CBuffer(void)
{
    ClearBuffer();
}


// destructor
CBuffer::~CBuffer(void)
{
}


int CBuffer::Read(char *pszBuffer, int iSize, int &iBytes)
{
    int iRetCode = RC_SUCCESS;
    int iLength;

    if(m_iLength > iSize)
    {
      iLength = iSize;
      iRetCode = RC_TRUNCATION_OCCURRED;
    }
    else
      iLength = m_iLength;
    memcpy(pszBuffer,m_szBuffer,iLength);
    iBytes = iLength;
    return(iRetCode);      
}


int CBuffer::WriteBuffer(char *pszBuffer, int iBytes)
{
    int iRetCode = RC_SUCCESS;
    int iLength;

    if(iBytes > m_iLength)
    {
      iLength = m_iLength;
      iRetCode = RC_TRUNCATION_OCCURRED;
    }
    else
      iLength = iBytes;
    memcpy(m_szBuffer,pszBuffer,iLength);
    return(iRetCode);      
}


int CBuffer::Write(char *pszFormat, ...)
{
    int iRetCode = RC_SUCCESS;
    va_list ap;

    va_start(ap, pszFormat);
    vsprintf(m_szBuffer,pszFormat,ap);
    va_end(ap);
    m_iLength = strlen(m_szBuffer);
    return(iRetCode);      
}


void CBuffer::ClearBuffer(void)
{
    memset(m_szBuffer,0,sizeof(m_szBuffer));
    m_iLength = 0;
}


int CBuffer::SetLength(int iLength)
{
    int iRetCode = RC_SUCCESS;

    if(iLength > MAX_BUFFERSIZE)
      iRetCode = RC_INVALID_PARAMETER;
    else
      m_iLength = iLength;
    return(iRetCode);
}

//---------------------------------------------------------------------
// end-of-file: csocket.cc                                             
//---------------------------------------------------------------------
