/*
 +---------------------------------------------------------------------+
 | upsstat.cc                                                          |
 | display ups info obtained from upsmond                              |
 |                                                                     |
 | Author:  Bob Pill, NCIS                                             |
 |          bobp@newcorridor.com                                       |
 |                                                                     |
 | Created: Sept 15, 1997                                              |
 |                                                                     |
 | Copyright (c) 1997 New Corridor Information Services.               |
 | All rights reserved.                                                |
 |                                                                     |
 |                                                                     |
 | THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR        |
 | IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED      |
 | WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR          |
 | PURPOSE.                                                            |
 |                                                                     |
 |---------------------------------------------------------------------|
 |  ***Revision-History***  Begin-Log                                  |
 |     Initial creation - (started at ver 2.0.1) Sept 15, 1997 (BP)    |
 |     Version 2.1.0 update Jan 01, 1998 (BP)                          |
 |     Version 2.1.1 update Feb 03, 1998 (BP)                          |
 |     Version 2.1.2 update May 17, 1998 (BP)                          |
 |     Version 2.1.3 update Aug 04, 1998 (BP)                          |
 |  ***Revision-History***  End-Log                                    |
 +---------------------------------------------------------------------+
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <unistd.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include "stddef.h"
#include "upsmonitor.h"


void Usage(char *pszProgramName)
{
    printf("\nusage: %s [-sh]\n",pszProgramName);
    printf("        [-h help] (display this help)\n");
    printf("        [-s socketname] (default is /tmp/upsmond.sock)\n\n");
}


void DisplayUpsInfo(UpsInfoResp *pResp)
{

 
    printf("\n        UPSMON Status Reporting Facility\n");

    if(pResp->iRetCode == RC_SUCCESS)
    {
      printf("        ---  upsmond version %s  ---\n\n",pResp->szUpsmondVersion);
      printf("             APC UPS 700 Status Info\n");
      printf("--------------------------------------------------------\n");
      printf("                     Model: %s\n",pResp->szModel);
      printf("                 CopyRight: %s\n",pResp->szCopyRight);
      printf("          Manufacture date: %s\n",pResp->szMfgDate);
      printf("                  Serial #: %s\n",pResp->szSerial);
      printf("          Firmware version: %s\n",pResp->szFirmware);
      printf("                  Local ID: %s\n",pResp->szLocalID);
      printf(" Last battery replace date: %s\n",pResp->szBattReplaceDate);
      printf("      Nominal line voltage: %d V\n",pResp->iNomLine);
      printf("   Nominal battery voltage: %d V\n",pResp->iNomBatt);
      printf("                 Available: %s\n",pResp->iAvailable ? "YES" : "NO");
      printf("                   Standby: %s\n",pResp->iStandby ? "YES" : "NO");
      printf("                Line Power: %s\n",pResp->iLine ? "OK" : "FAILURE");
      printf("                   Battery: %s\n",pResp->iLowBatt ? "LOW" : "OK");
      printf("              Line Voltage: %6.2f\n",pResp->dLineVoltage);
      printf("                      Load: %6.2f %%\n",pResp->dLoad);
      printf("           Battery Voltage: %6.2f\n",pResp->dBattVoltage);
      printf("             Battery Level: %6.2f %%\n",pResp->dBatteryLevel);
      printf("--------------------------------------------------------\n");
    }
    else
      printf("\n\nupsmond response error: %d\n\n",pResp->iRetCode);
}



int main(int argc, char **argv)
{
    int iRetCode = RC_SUCCESS;
    int iErrors = 0;
    int iOption = 0;
    CSocket sockClient;
    CBuffer *pbufSend = NULL;
    CBuffer *pbufRcve = NULL;
    UpsInfoReq *pReq = NULL;
    UpsInfoResp *pResp = NULL;
    char *pszSlash = NULL;
    char szProgramName[MAX_PATHNAME];
    char szSocketName[MAX_PATHNAME];

    // initialization
    memset(szProgramName,0,sizeof(szProgramName));
    memset(szSocketName,0,sizeof(szSocketName));
    strcpy(szSocketName,"/tmp/upsmond.sock");
    if((pszSlash = strrchr(argv[0], '/')))
      strcpy(szProgramName,pszSlash+1);
    else
      strcpy(szProgramName,argv[0]);
    
    // parse command line options
    while((iOption = getopt(argc, argv, "hs:")) != EOF)
    {
      switch(iOption)
      {
        case 'h':
          Usage(szProgramName);
          goto MainExit;
          break;

        case 's':
          strcpy(szSocketName,optarg);
          printf("\nSocketName set to: %s\n",szSocketName);
          break;

        case '?':
          iErrors++;
      }
    }

    if(iErrors)
    {
      Usage(szProgramName);
      iRetCode = RC_INVALID_CMDLINE_ARGS;
      goto MainExit;
    }

    iRetCode = sockClient.Connect(szSocketName);
    if(iRetCode == RC_SUCCESS)
    {
      pbufSend = sockClient.GetSendBuffer();
      pbufRcve = sockClient.GetRcveBuffer();
      pReq = (UpsInfoReq *)(pbufSend->GetBuffer());
      memset(pReq,0,sizeof(UpsInfoReq));
      pReq->iMsgFormatID = UPSInfoMsgFormatID;
      pReq->iFunctionID = UPS_GET_STATUS;
      pbufSend->SetLength(sizeof(UpsInfoReq));
      iRetCode = sockClient.Write();
      if(iRetCode == RC_SUCCESS)
      {
        iRetCode = sockClient.Read();
        if(iRetCode == RC_SUCCESS)
        {
          pResp = (UpsInfoResp *)pbufRcve->GetBuffer();
          DisplayUpsInfo(pResp);
        }
        else
          printf("Read error on upsmond socket: %d\n",iRetCode);
      }
      else
        printf("Write error on upsmond socket: %d\n",iRetCode);
      sockClient.Release();
    }
    else
      printf("Could not connect to upsmond: %d\n",iRetCode);

MainExit:

    return(iRetCode);
}
